// $URL: file:///usr/local/subversion/projects_svn/packages/progresswindow/branches/IGOR-6-20--1/Progress Window.ipf $
// $Author: rgerkin $
// $Rev: 1340 $
// $Date: 2010-07-01 09:38:06 -0700 (Thu, 01 Jul 2010) $

// Simple Usage:  
// ProgWinOpen() creates a new progress panel.  
// ProgWin(1/2,0) sets the topmost bar to be 1/2 filled.  
// ProgWin(3/11,1) sets the second bar (indexing from 0) to be 3/11 filled.  
// ProgWin(-1,1) kills the second bar.  
// ProgWin(1/2,0,status="Analyzing") sets the topmost bar to be 1/2 filled with a status message next to it that says "Analyzing".

// Example:  
// svar dataList,taskList // Pretend that these are some global variables listing your data files and a list of functions to perform on each data file.  
// variable i
// variable numData=itemsinlist(dataList)
// variable numTasks=itemsinlist(taskList)
// ProgWinOpen()
// for(i=0;i<numData;i+=1)
//   string data=stringfromlist(i,dataList)
//   ProgWin(i/numData,0,status=data)
// 	 for(j=0;j<numTasks;j+=1)
//		string task=stringfromlist(i,taskList)
//		ProgWin(j/numTasks,1,status=task)
//		DoSomeTask(data,task)
//	 endfor
// endfor
// ProgWinClose()

#pragma rtGlobals=1		// Use modern global access method.

constant bar_height=30
constant bar_width=250
constant max_depth=10

Function ProgWinOpen([name,left,top,width,height])
	String name
	variable left,top,width,height
	
	Variable new=0
	if(!DataFolderExists("root:Packages:ProgWin"))
		NewDataFolder /O root:Packages
		NewDataFolder /O root:Packages:ProgWin
		String /G root:Packages:ProgWin:winNames=""
		Variable /G root:Packages:ProgWin:deep=0
	endif
	SVar winNames=root:Packages:ProgWin:winNames
	NVar deep=root:Packages:ProgWin:deep
	
	if(ParamIsDefault(name))
		name="ProgressWin"
	endif
	string title=name
	
	name=CleanupName(name,0)
	winNames+=name+";"
	
	if(WinType(name))	
		DoWindow /F $name
	else
		left=paramisdefault(left) ? 100 : left
		top=paramisdefault(top) ? 100 : top
		width=paramisdefault(width) ? 100+bar_width : width
		height=paramisdefault(height) ? 25+bar_height : height
		NewPanel /K=1 /N=$name /W=(left,top,left+width,top+height) as title
		MoveWindow /W=$name left,top,left+width,top+height
	endif
	DoUpdate /W=$name /E=1
	Button Abort_0, pos={bar_width+10,30}, title="Abort", proc=ProgWinButtons
	SetWindow $name userData(deep)="1",hook(CloseWindow)=ProgWinCloseHook
	ProgWin(0,0,name=name)
End

Function ProgWinCloseHook(info)
	STRUCT WMWinHookStruct &info
	if(info.eventCode==2) // Window killed.  
		String name=info.winName
		ProgWinClose(name=name)
	endif
End

Function ProgWin(frac,depth[,status,name])
	Variable frac,depth
	String status,name
	if(!DataFolderExists("root:Packages:ProgWin"))
		ProgWinOpen()
		//print "No ProgWin data folder in root:Packages"
		//return 0
	endif
	SVar winNames=root:Packages:ProgWin:winNames
	if(ParamIsDefault(name))
		name=StringFromList(0,winNames)
	endif
	name=CleanupName(name,0)
	if(WhichListItem(name,winNames)<0)
		print "No Progress Window with name '"+name+"'"
		return 0
	endif
	Variable deep=str2num(GetUserData(name,"","deep"))
	ControlInfo /W=$name $("Prog"+num2str(depth))
	if(!V_flag)
		ValDisplay $("Prog"+num2str(depth)),pos={80,30+(bar_height*1.35*depth)},size={bar_width,bar_height*2/3},limits={0,1,0},barmisc={0,0}, mode=3, win=$name
		TitleBox $("Status"+num2str(depth)), pos={5,30+(bar_height*1.35*depth)},size={60,bar_height*2/3}, win=$name
	elseif(frac<0)
		KillControl /w=$name $("Prog"+num2str(depth))
		KillControl /w=$name $("Status"+num2str(depth))
		return 0
	endif
	ValDisplay $("Prog"+num2str(depth)),value=_NUM:frac, win=$name
	if(!ParamIsDefault(status))
		TitleBox $("Status"+num2str(depth)), title=status, win=$name
	endif
	if(depth>=deep)
		MoveWindow /W=$name 600,100,610+bar_width,125+bar_height*(depth+1)
		SetWindow $name userData(deep)=num2str(depth+1)
	endif
	DoUpdate /W=$name /E=1
	NVar /Z abortDepth=root:Packages:ProgWin:$("abortDepth_"+name)
	if(NVar_Exists(abortDepth))
		if(depth<=abortDepth)
			KillVariables abortDepth
			if(depth==0)
				//ProgWinClose(name=name)
			endif
			return 1 // Abort code.  
		endif
	else
		return 0
	endif
End

Function ProgWinClose([name])
	String name
	if(DataFolderExists("root:Packages:ProgWin"))
		SVar winNames=root:Packages:ProgWin:winNames
		NVar deep=root:Packages:ProgWin:deep
		deep-=1
	else
		print "No ProgWin data folder in root:Packages"
		return 0
	endif	
	if(ParamIsDefault(name))
		name=StringFromList(0,winNames)
	endif
	name=CleanupName(name,0)
	Variable windows_left=ItemsInList(winNames)
	DoWindow /K $name
	if(windows_left<=1)
		KillDataFolder /Z root:Packages:ProgWin
	else
		winNames=RemoveFromList(name,winNames)
	endif
End

Function ProgWinButtons(ctrlName)
	String ctrlName
	
	Variable button_num
	String action=StringFromList(0,ctrlName,"_")
	Variable depth=str2num(StringFromList(1,ctrlName,"_"))	
	strswitch(action)
		case "Abort":
			String name=WinName(0,64)
			ProgWinAbort(depth,name=name)
	endswitch
End

// Generic progress window abort procedure.  
Function ProgWinAbort(depth[,name])
	Variable depth
	String name
	Variable /G root:Packages:ProgWin:$("abortDepth_"+name)=depth
End

Function ProgWinKill()
	if(!DataFolderExists("root:Packages:ProgWin"))
		return -1
	endif	
	SVar winNames=root:Packages:ProgWin:winNames
	Variable i,numWinNames=ItemsInList(winNames)
	for(i=0;i<numWinNames;i+=1)
		if(SVar_Exists(winNames))
			String name=StringFromList(i,winNames)
			DoWindow /K $name
		endif
	endfor
	KillDataFolder /Z root:Packages:ProgWin
End
